﻿using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Layouts;
using ArcGIS.Desktop.Mapping;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace CIMGraphic
{
    internal class Module1 : Module
    {
        private static Module1 _this = null;

        /// <summary>
        /// Retrieve the singleton instance to this module here
        /// </summary>
        public static Module1 Current
        {
            get
            {
                return _this ?? (_this = (Module1)FrameworkApplication.FindModule("CIMGraphic_Module"));
            }
        }

        public static Geometry GetGeometry (string layerName, string whereClause)
		{
			var theLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().FirstOrDefault(f => f.Name.Equals(layerName, StringComparison.CurrentCultureIgnoreCase));
			if (theLayer == null) return null;
			Geometry result = null;
			var fcSource = theLayer.GetTable() as FeatureClass;
			if (fcSource == null) return null;
			var qf = new QueryFilter()
			{
				WhereClause = whereClause
			};
			using (var cursor = fcSource.Search(qf))
			{
				if (cursor.MoveNext())
				{
					using (var parcelFeature = cursor.Current as Feature)
					{
						result = parcelFeature.GetShape().Clone();
					}
				}
			}
            return result;
		}

        #region Overrides
        /// <summary>
        /// Called by Framework when ArcGIS Pro is closing
        /// </summary>
        /// <returns>False to prevent Pro from closing, otherwise True</returns>
        protected override bool CanUnload()
        {
            //TODO - add your business logic
            //return false to ~cancel~ Application close
            return true;
        }

        #endregion Overrides

    }
}
